/*
  CatWeasel MK3 PCI Linux driver (c) 2002 by Dirk Jagdmann <doj@cubic.org>

  This file is part of CatWeasel MK3 PCI Linux driver.

  The CatWeasel MK3 PCI Linux driver is free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  The CatWeasel MK3 PCI Linux driver is distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with the CatWeasel MK3 PCI Linux driver; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  02111-1307 USA
  $Header: /home/cvs/sidblaster/device/cwpci.c,v 1.3 2003/05/20 22:19:41 doj Exp $
*/

/*
  Code for finding, initializing and registering the card
*/

#include <cw.h>

#include <linux/pci.h>

int cw_num;
int cw_register[CWSID_MAX_NUM][RegisterNum];

static int cw_iobase[CWSID_MAX_NUM];

#if 0
static void cw_interupt(int irq, void *id, struct pt_regs *regs)
{
  wake_up_interruptible(&sid_queue);
}
#endif

static int __devinit cw_probe(struct pci_dev *pcidev, const struct pci_device_id *pciid)
{
  int err;

  if(cw_num+1>=CWSID_MAX_NUM)
    {
      printk(KERN_ERR DEVICE_NAME ": found another card, but maximum number (%i) of CatWeasel cards detected.\n", CWSID_MAX_NUM);
      return -1;
    }

  if((err=pci_enable_device(pcidev)))
    {
      printk(KERN_ERR DEVICE_NAME ": could not enable device\n");
      return err;
    }

  cw_iobase[cw_num] = pci_resource_start(pcidev, 0);

  if(!request_region(cw_iobase[cw_num], 256, DEVICE_NAME)) 
    {
      printk(KERN_ERR DEVICE_NAME ": io ports 0x%04x-0x%04x in use\n", cw_iobase[cw_num], cw_iobase[cw_num]+255);
      return -EBUSY;
    }

  printk(KERN_NOTICE DEVICE_NAME ": found a CatWeasel at 0x%04X\n", cw_iobase[cw_num]);

  cw_register[cw_num][JoyDat]=cw_iobase[cw_num]+0xc0;
  cw_register[cw_num][PaddleSelect]=cw_iobase[cw_num]+0xc4;
  cw_register[cw_num][JoyButton]=cw_iobase[cw_num]+0xc8;
  cw_register[cw_num][JoyButtondir]=cw_iobase[cw_num]+0xcc;
  cw_register[cw_num][KeyDat]=cw_iobase[cw_num]+0xd0;
  cw_register[cw_num][KeyStatus]=cw_iobase[cw_num]+0xd4;
  cw_register[cw_num][SidDat]=cw_iobase[cw_num]+0xd8;
  cw_register[cw_num][SidCommand]=cw_iobase[cw_num]+0xdc;
  cw_register[cw_num][CatMem]=cw_iobase[cw_num]+0xe0;
  cw_register[cw_num][CatAbort]=cw_iobase[cw_num]+0xe4;
  cw_register[cw_num][CatControl]=cw_iobase[cw_num]+0xe8;
  cw_register[cw_num][CatOption]=cw_iobase[cw_num]+0xec;
  cw_register[cw_num][CatStartA]=cw_iobase[cw_num]+0xf0;
  cw_register[cw_num][CatStartB]=cw_iobase[cw_num]+0xf4;
  cw_register[cw_num][CatIRQ]=cw_iobase[cw_num]+0xfc;

#if 1
  outb(0xf1, cw_iobase[cw_num] + 0x0);
  outb(0x00, cw_iobase[cw_num] + 0x1);
  outb(0x00, cw_iobase[cw_num] + 0x2);
  outb(0x00, cw_iobase[cw_num] + 0x4);
  outb(0x00, cw_iobase[cw_num] + 0x5);
  outb(0x00, cw_iobase[cw_num] + 0x29);
  outb(0x00, cw_iobase[cw_num] + 0x2b);
#endif

  cw_num++;
  return 0;
}

static __devinitdata struct pci_device_id id_table[] = {
  { 0xe159, 0x0001, 0x1212, 0x0002, 0, 0 },
  { 0 }
};

MODULE_DEVICE_TABLE(pci, id_table);

static struct pci_driver cw_driver = {
  name: "cw",
  id_table: id_table,
  probe: cw_probe,
};

static void __exit cw_exit()
{
  int i;

  cwsid_exit();
  cwjoy_exit();

#if 0
  free_irq(sid_irq, sid_id);
#endif

  for(i=0; i<cw_num; i++)
    release_region(cw_iobase[i], 256);
  pci_unregister_driver(&cw_driver);
  printk (KERN_INFO DEVICE_NAME " uninstalled\n");
}

static int __init cw_init()
{
  int err;

  cw_num=0;
  for(err=0; err<CWSID_MAX_NUM; err++)
    {
      int i;
      for(i=0; i<RegisterNum; i++)
	cw_register[err][i]=-1;
      cw_iobase[err]=-1;
    }

  if(pci_present()) 
    {
      err = pci_module_init(&cw_driver);
      if(err)
	return err;
    }

  err=cwsid_init();
  if(err<0)
    {
      cw_exit();
      return err;
    }

  err=cwjoy_init();
  if(err<0)
    {
      cw_exit();
      return err;
    }

  printk (KERN_INFO DEVICE_NAME " installed\n");
  return 0;
}

module_init(cw_init);
module_exit(cw_exit);
EXPORT_NO_SYMBOLS;

MODULE_AUTHOR("(c) 2002-2003 Dirk Jagdmann <doj@cubic.org> http://llg.cubic.org/cw");
MODULE_DESCRIPTION("device driver for CatWeasel MK3 PCI");
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,17)
MODULE_LICENSE("GPL");
#endif
