/*
  sid (c) 1999-2003 by Dirk Jagdmann <doj@cubic.org>

  This file is part of sid.

  sid is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  sid is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
  License for more details.

  You should have received a copy of the GNU General Public License
  along with sid; if not, write to the Free Software Foundation, Inc.,
  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  
  $Header: /home/cvs/sidblaster/sid/configuration.cpp,v 1.4 2003/05/31 10:34:00 doj Exp $
*/

#include "configuration.hpp"
#include <fstream>
#include <locale>
#include <iostream>
using namespace std;

void doj::convToLower(std::string& s)
{
  locale loc;
  for(size_t i=0; i<s.size(); i++)
    s[i]=tolower(s[i], loc);
}

doj::Configuration::Configuration(const std::string& fn) :
  filename(fn)
{
  ifstream is(filename.c_str());
  while(is)
    {
      string line;
      getline(is, line);

      // strip comments
      size_t pos=line.find('#');
      if(pos!=string::npos)
	line.erase(pos);

      istringstream iss(line);
      string key, value, s;
      bool keyflag=false, valueflag=false;
      while(iss)
	{
	  iss >> s;
	  // check for EOL
	  if(!iss)
	    goto nextline;

	  string k=s;

	  // check for separator
	  pos=s.find('=');
	  if(pos==string::npos)
	    pos=s.find(':');
	  if(pos!=string::npos)
	    {
	      k.erase(pos);
	      s.erase(0, pos+1);
	      // strip whitespace from beginning
	      pos=s.find_first_not_of(" \t");
	      if(pos>0)
		s.erase(0, pos);
	    }
	  else
	    s.erase();

	  if(keyflag)
	    key+=' ';
	  key+=k;
	  keyflag=true;
	}

      while(iss || s.size())
	{
	  if(s.empty())
	    iss >> s;

	  if(s.size())
	    {
	      if(valueflag)
		value+=' ';
	      value+=s;
	      valueflag=true;

	      s.erase();
	    }
	}

      if(key.size() /*&& value.size()*/)
	{
	  convToLower(key);
	  m[key]=value;
	}
    nextline: ;
    }


  std::cout << "conf:" << endl;
  for(map_t::const_iterator i=m.begin(); i!=m.end(); ++i)
    std::cout << i->first << " = " << i->second << std::endl;
}

doj::Configuration::~Configuration()
{
}

bool doj::Configuration::getBool(std::string key)
{
  if(!isIncluded(key))
    return false;

  convToLower(key);
  string s=m[key];
  convToLower(s);
  if(s=="true" || s=="1" || s=="yes")
    return true;
  return false;
}

int doj::Configuration::getInt(std::string key)
{
  if(!isIncluded(key))
    return 0;

  convToLower(key);
  istringstream is(m[key]);
  int i=0;
  is >> i;
  return i;
}

double doj::Configuration::getDouble(std::string key)
{
  if(!isIncluded(key))
    return 0;

  convToLower(key);
  istringstream is(m[key]);
  double d=0;
  is >> d;
  return d;
}

void doj::Configuration::save() const
{
  ofstream os(filename.c_str());
  if(os)
    {
      for(map_t::const_iterator i=m.begin(); i!=m.end(); ++i)
	os << i->first << " = " << i->second << endl;
    }
}

void doj::Configuration::updateString(std::string key, std::string& s)
{
  convToLower(key);
  if(isIncluded(key))
    s=m[key];
}

void doj::Configuration::updateString(std::string key, char* &s)
{
  convToLower(key);
  if(isIncluded(key))
    s=strdup(m[key].c_str());
}

void doj::Configuration::updateBool(std::string key, bool& b)
{
  convToLower(key);
  if(isIncluded(key))
    b=getBool(key);
}

void doj::Configuration::updateDouble(std::string key, double& d)
{
  convToLower(key);
  if(isIncluded(key))
    d=getDouble(key);
}

bool doj::Configuration::isIncluded(std::string key) const
{ 
  convToLower(key);
  return m.find(key)!=m.end();
}

void doj::Configuration::erase(std::string key)
{
  convToLower(key);
  m.erase(key); 
}

std::string doj::Configuration::getString(std::string key)
{
  if(!isIncluded(key))
    return "";
  convToLower(key);
  return m[key];
}
