/*
  sid (c) 1999-2003 by Dirk Jagdmann <doj@cubic.org>

  This file is part of sid.

  sid is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  sid is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
  License for more details.

  You should have received a copy of the GNU General Public License
  along with sid; if not, write to the Free Software Foundation, Inc.,
  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  
  $Header: /home/cvs/sidblaster/sid/configuration.hpp,v 1.3 2003/05/21 23:10:32 doj Exp $
*/

#ifndef CONFIGURATION__HPP
#define CONFIGURATION__HPP

#include <map>
#include <string>
#include <sstream>

namespace doj
{
  
  /// convert s to lowercase
  void convToLower(std::string& s);

  /**
     A class to manage a configuration file. The configuration file is
     loaded with the constructor and can be queued by various
     functions with various types. Keys or values can be altered and
     saved again. All Keys are accessed NOT CASESENSITIV.
  */
  class Configuration
  {
  protected:
    /// container type used to store keys and values
    typedef std::map<std::string, std::string> map_t;

    /// container for keys and values
    map_t m;

    /// filename of configuration file
    std::string filename;

  public:
    /// use filename with this instance
    Configuration(const std::string& filename);
    virtual ~Configuration();

    /// save configuration file
    void save() const;

    /// store key/value in the configuration
    template <typename T>
    void set(std::string key, const T& value)
    {
      convToLower(key);
      std::ostringstream os;
      os << value;
      m[key]=os.str();
    }

#if 0
    /// store key/value in the configuration
    template <>
    void set(std::string key, const std::string& value)
    { 
      convToLower(key);
      m[key]=value; 
    }
#endif

    /// @return true if key is included in the configuration. false otherwise
    bool isIncluded(std::string key) const;

    /// remove key from configuration
    void erase(std::string key);

    /// @return value for key. if key is not included in the configuration an empty string is returned.
    std::string getString(std::string key);
    /// @return boolean for key. if key is not included in the configuration false is returned.
    bool getBool(std::string key);
    /// @return integer for key. if key is not included in the configuration 0 is returned.
    int getInt(std::string key);
    /// @return double for key. if key is not included in the configuration 0 is returned.
    double getDouble(std::string key);

    /// if key is included in the configuration s is set to key's value.
    void updateString(std::string key, std::string& s);
    /// if key is included in the configuration s is set to key's value.
    void updateString(std::string key, char* &s);
    /// if key is included in the configuration b is set to key's value.
    void updateBool(std::string key, bool& b);
    /// if key is included in the configuration d is set to key's value.
    void updateDouble(std::string key, double& d);
    /// if key is included in the configuration i is set to key's value. i should be an integer type or have a conversion from int.
    template <typename T>
    void updateInt(std::string key, T& i)
    {
      if(isIncluded(key))
	i=getInt(key);
    }

  private:
    Configuration();
    Configuration(const Configuration&);
    Configuration& operator=(const Configuration&);

  };

};

#endif
